/*
 -  X V P O R T. C
 -
 *  Purpose:
 *      This file contains the implemenation
 *      of the eXpandable Viewport tool
 *
 *
 *
 */

#include <windows.h>
#include <windowsx.h>
#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>
#include <ctype.h>

#ifdef WIN16
#include <compobj.h>
#include <storage.h>
#include <commdlg.h>
#endif

#include <pvalloc.h>
#include <strtbl.h>
#include "_xvport.h"
#include "resource.h"


// Library globals
DLGPROC   	dlgproc;	  		  //dialog procedure instance
UINT      	GuID = 0;	  		  //Viewport id
HINSTANCE 	GhInst;	  			  //library instance handle
HWND      	GhParent;	  		  //Parent window handle
LPVP      	VPTail = NULL;		  //Viewport item list Tail
LPVP      	VPHead = NULL;		  //Viewport item list Head
HCURSOR 	hWaitCursor;		  //Wait cursor
HCURSOR 	hNormalCursor;		  //Normal cursor
BOOL 		GbIsInit=FALSE;		  //Library initialized
BOOL        bMultiThread = FALSE; //Multithreading enabled

#ifdef _WIN32    
CRITICAL_SECTION cs;	    	  //Critical Section

/*
 -  LibMain() Version Win32
 -
 *  Purpose:
 *      Initialization and DeInitialization
 *      for the library instance
 *      Init 
 *	   		Save the instance handle in a global
 *	    	variable for later use
 *	   	DeInit
 *			DeInitialize viewport memory
 *	   
 */

BOOL WINAPI
DllMain( HINSTANCE hInstance,
         DWORD     fdwReason,
         LPVOID    lpvReserved)

{
    switch(fdwReason)
    {
        case  DLL_PROCESS_ATTACH:


            GhInst = hInstance;
            break;
        case DLL_PROCESS_DETACH:
            XVPDeInit();
			UnregisterClass("XVPDLGCLASS", GhInst);
			if(bMultiThread)
				DeleteCriticalSection(&cs);
            break;
        default:
            break;
    }
    return(TRUE);

}

#else

/*
 -  LibMain() Version Win16
 -
 *  Purpose:
 *      Initialization for the library instance
 *       -Init 
 *	   		Save the instance handle in a global
 *	    	variable for later use
 *	   
 */

int FAR PASCAL LibMain(HANDLE hInstance,
                       WORD wDataSeg,
                       WORD wHeapSize,
                       LPSTR lpszCmdLine)
{
    if(wHeapSize > 0)
        UnlockData(0);
    GhInst = hInstance;


    return(1);

}

/*
 -  _WEP() Version Win16
 -
 *  Purpose:
 *      DeiInitialization for the library instance
 *      -DeInit
 *			DeInitialize viewport memory
 *	   
 */

int FAR PASCAL _WEP(int n)
{
   	//Destroy viewport all viewports         
    XVPDeInit();
    return 1;
}

#endif


/*
 -  XVPInit()
 -
 *  Purpose:
 *      Initialization for the viewport system
 *	    Register a window class
 *	   
 */


BOOL    XVPInit(HWND hParent )
{

    WNDCLASS    wc;
    int         d = 0;

    GhParent = hParent;
	if(!GhParent)
		 GhParent = GetDesktopWindow();

    //If we are inited already simply return
    if(GbIsInit)
        return TRUE;


        
    //Register the Window class if neccessary

    wc.style         =  0;
    wc.lpfnWndProc   =  DefDlgProc;             //The default Dlg Proc
    wc.cbClsExtra    =  0;
    wc.cbWndExtra    =  DLGWINDOWEXTRA;         //VIP for Dlg or it won't work
    wc.hInstance     =  GhInst;
    wc.hIcon         =  LoadIcon(GhInst,MAKEINTRESOURCE(IDR_XVPORT));
    wc.hCursor       =  0 ;
    wc.hbrBackground =  (HBRUSH) GetStockObject(LTGRAY_BRUSH);
    wc.lpszMenuName  =  MAKEINTRESOURCE(IDR_MENU);
    wc.lpszClassName =  "XVPDLGCLASS";

    d = RegisterClass(&wc);
        
    hWaitCursor = LoadCursor((HINSTANCE)NULL, IDC_WAIT);
    GbIsInit = TRUE;
    return TRUE;

}

/*
 -  XVPInitMultiThread()
 -
 *  Purpose:
 *      Initializes multithreading
 *	    The first viewport is the only one
 *		that can really set up for multithreading
 *
 */

BOOL XVPInitMultiThread()
{

#ifdef _WIN32

	bMultiThread = TRUE;

	//Initialize critical section
    InitializeCriticalSection(&cs);

	return TRUE;

#endif

	return FALSE;
}
/*
 -  XVPCreateViewport()
 -
 *  Purpose:
 *      Create a viewport
 *	     - Allocate viewport structures
 *		 - Load options from ini file
 *	   	 - Create Viewport dialog
 */

UINT XVPCreateViewport(LPSTR szTitle)
{

    LPTD     lptd = NULL;
    LPVP     lpvp = NULL;

	//Init if not already inited
    //just  return
	if(!GbIsInit)
        return 0;

    //Allocate a threaddata structure
    lptd = (LPTD)GlobalAllocPtr(GHND, sizeof(TD));
    if(!lptd)
    {
        MessageBox(GhParent, "Error Allocating Viewport memory.", 
        		   "Error", MB_OK | MB_ICONSTOP);
        return 0;
    }


    //Allocate a viewport structure
    lpvp = (LPVP)GlobalAllocPtr(GHND,sizeof(VP));
    if(!lpvp)
    {
        MessageBox(GhParent, "Error Allocating Viewport memory.", 
        		   "Error", MB_OK | MB_ICONSTOP);
        return 0;
    }

    //Now fill in the structure
    lpvp->uID  		 	= 	++GuID;
    lpvp->lptd 		 	= 	lptd;
    lpvp->lpItemHead 	= 	NULL;
    lpvp->lpItemTail 	= 	NULL;
    lpvp->next       	= 	NULL;
	lstrcpy(lpvp->szTitle, szTitle);
	
	//Load profile settings from ini file
    LoadProfileSettings( lpvp);

    //Now chain this structure to the global list of viewports
    if(!VPHead)
    {
        VPHead = lpvp;
        VPTail = VPHead;
    }
    else
    {
        VPTail->next = lpvp;
        VPTail       = lpvp;
    }

   //Now create the window for the Viewport

	if(bMultiThread)
	{
#ifdef _WIN32
		
		//For _WIN32 create a thread and use a modal dialog
		lptd->hEvent = CreateEvent(NULL, TRUE, FALSE, NULL);
		lptd->hT     = CreateThread(NULL, 0, 
									(LPTHREAD_START_ROUTINE)ViewPortThread,
    	                            lpvp, 0, &lptd->dwTId);
    
		if(!lptd->hT)
    	{
    	    MSGSTOP("Error creating Viewport thread");
    	    return 0;
    	}
    
		//Wait for the signaling of the Manual reset event
		//This will be when the modal dialog is succesfully 
    	//created
    	WaitForSingleObject(lptd->hEvent, INFINITE);
    
    	if(!lptd->hDlg)
    	{
    		MSGSTOP("Error creating Viewport Dialog");
 		    return 0;
    
    	}
    
#endif
	} 
	else
	{
		//For WIN16 create a modeless dialog
    	dlgproc = (DLGPROC)MakeProcInstance(ViewPortDlgProc, GhInst);

    	lptd->hParent    = GhParent;
    	lpvp->lptd->hDlg = CreateDialogParam(GhInst, "XViewPort", GhParent,
    										 dlgproc, (LPARAM) lpvp);
    	if(!lptd->hDlg)
    	{
 			MSGSTOP("Error creating Viewport Dialog");
    	    return 0;
    	}

		//Show the modeless dialog
    	ShowWindow(lptd->hDlg, SW_SHOW);
    	UpdateWindow(lptd->hDlg);


	}
	
	// Now that we have a window we can set it's title
    SetWindowText(lpvp->lptd->hDlg,szTitle);

	//return a magic cookie to the caller
	return lpvp->uID;

}


/*
 -  XVPDestroyViewport()
 -
 *  Purpose:
 *      Destroy a viewport
 *	     - DeAllocate viewport structures
 *		 - Save options to ini file
 */


VOID   XVPDestroyViewport(UINT uID)
{
    LPVP lpT1;
    LPVP lpT2;
    int  iTemp = 0;

    //Find this viewport structure in the
    //global viewport list and remove it
    if(!VPHead)
        return;

    if(VPHead->uID == uID)
    {
        lpT1 = VPHead;

         //Free Item List
        FreeVPIList(uID);
		        
        goto Free;
    }

    for(lpT2 = VPHead; lpT2->next && lpT2->next->uID != uID ; lpT2 = lpT2->next);

    //If we didn't find it
    if(!lpT2->next )
        return;

    //We found it now  remove it
    lpT1  = lpT2->next;

     //Free Item List before you unchain it
    FreeVPIList(uID);

    lpT2->next = lpT1->next;

    //If we are freeing the tail we need
    //to set the Tail ptr to the new last Item
    if(lpT1== VPTail)
        VPTail = lpT2;

Free:

	//Save Profile Settings
    SaveProfileSettings(lpT1);
	
    //Destroy the Dlg window
    SendDlgItemMessage(lpT1->lptd->hDlg, IDC_VIEWPORT, LB_RESETCONTENT, 0, 0);
    SendMessage(lpT1->lptd->hDlg, WM_CLOSE, 0, 0);

    //EndDialog(lpT1->lptd->hDlg, TRUE);


    //Free the threaddata structure
    GlobalFreePtr (lpT1->lptd);


	//Move the Head pointer to the next
	//Before freeing
	VPHead = VPHead->next;

    //Free the  vp structure
    GlobalFreePtr(lpT1);

}

/*
 -  XVPDInit()
 -
 *  Purpose:
 *      DeInitialize the viewport system
 *	     - Free the list of active viewports
 */


VOID XVPDeInit( )
{

    if(!GbIsInit)
        return;

    //Traverse the VP List and Reset all viewports
    while(VPHead)
    {
        XVPDestroyViewport(VPHead->uID );

    }

    
    GbIsInit = FALSE;
}


/*
 -  XVPSetTitle()
 -
 *  Purpose:
 *      Set  title for viewport
 */


VOID XVPSetTitle(UINT uID, LPSTR szTitle )
{
    LPVP lpvp;
    lpvp = VPFromID(uID);
	if(!lpvp) return;

    SetWindowText(lpvp->lptd->hDlg,szTitle);
    lstrcpy(lpvp->szTitle, szTitle);
}


/*
 -  XVPSetTitle()
 -
 *  Purpose:
 *      Invert the viewport display mode
 */


VOID   XVPInvert(UINT uID)
{
    LPVPITEM 	lpT1;
    LPVP 		lpvp;
    HWND 		hDlg;

    lpvp = VPFromID(uID);
    hDlg = lpvp->lptd->hDlg;

    if(!lpvp || !lpvp->lpItemHead)
        return;

	//Switch the inversion mode
    lpvp->fInversion = !lpvp->fInversion;
    
    //Clear the viewport content
    SendDlgItemMessage(hDlg, IDC_VIEWPORT, WM_SETREDRAW,   FALSE, 0);
    SendDlgItemMessage(hDlg, IDC_VIEWPORT, LB_RESETCONTENT,0,     0);
    
    //Go thru the list of items and restore list box
    //Inversion is effected by either inserting or
    //simply adding
    for(lpT1 = lpvp->lpItemHead; lpT1; lpT1 = lpT1->next)
    {
        
        if(lpT1->IsInBox)
                if(lpvp->fInversion)
                    SendDlgItemMessage(hDlg,
                                       IDC_VIEWPORT,
                                       LB_INSERTSTRING, 0, (LPARAM)lpT1);
                else
                    SendDlgItemMessage(hDlg,
                                       IDC_VIEWPORT,
                                       LB_ADDSTRING, 0, (LPARAM)lpT1);
    }
    SendDlgItemMessage(hDlg, IDC_VIEWPORT, WM_SETREDRAW,   TRUE, 0);

}

/*
 -  XVPShow()
 -
 *  Purpose:
 *      Invert the viewport display mode
 */

VOID   XVPShow(UINT uID, int nShow)
{
    int 	nSysShow;
    LPVP 	lpvp;
    HWND 	hDlg;

    lpvp = VPFromID(uID);
	if(!lpvp) return;

    hDlg = lpvp->lptd->hDlg;

    switch(nShow)
    {
        case XVP_HIDE:
            nSysShow  = SW_HIDE;
            break;
        case XVP_SHOW:
            nSysShow  = SW_SHOW;
            break;
        case XVP_MIN:
            nSysShow  = SW_SHOWMINIMIZED;
            break;
        case XVP_MAX:
            nSysShow  = SW_SHOWMAXIMIZED;
            break;
        default:
            return;
     }
     ShowWindow(hDlg, nSysShow);

}

/*
 -  XVPLog()
 -
 *  Purpose:
 *      Log a string to the viewport
 */

VOID XVPLog(UINT uID, int nLevel, LPSTR lpDesc)
{

  	XVPLogStatus(uID, nLevel, lpDesc, HEAD);

}


/*
 -  XVPLogStatus()
 -
 *  Purpose:
 *      Log a string to the viewport
 *      indicating Success or Failure
 *      
 */

VOID   XVPLogStatus(UINT uID,  int nLevel, LPSTR lpDesc, int  nType)
{
    static 		int n ;
    LPVP 		lpvp 	= NULL;
    HWND 		hDlg;
    LPSTR       lpBuff  = NULL;
    LPVPITEM    lpVPI   = 0;
    int         nret    = 0;
        
    lpvp 	= VPFromID(uID);
	if(!lpvp) return;

    hDlg	= lpvp->lptd->hDlg;
    nLevel 	= max(nLevel, 0);

	//We need 3 extra bytes. Two for the marker character
	//and 1 for the end of string character
	lpBuff  = (LPSTR)GlobalAllocPtr(GHND,strlen(lpDesc) + 3 );
    if(!lpBuff)
    	return;
        
    //Allocate and initialize new item
    lpVPI  = (LPVPITEM)GlobalAllocPtr(GHND, sizeof(VPITEM));
    if(!lpVPI)
    	return;
        
    lpVPI->sz     = lpBuff;
    lpVPI->nType  = nType;
    lpVPI->nLevel = nLevel;
    lpVPI->IsOpen = nLevel < lpvp->nLogLevel ? TRUE : FALSE;
    lpVPI->IsInBox= FALSE;
    lpVPI->next   = NULL;


    //Append the new item to the list;
    if(!lpvp->lpItemHead)
    {
        lpvp->lpItemHead = lpVPI;
        lpvp->lpItemTail = lpvp->lpItemHead;
    }
    else
    {
        lpvp->lpItemTail->next = lpVPI;
        lpvp->lpItemTail       = lpVPI;
    }

	//if we didn't get a proper item or a proper
    //string buffer we simply return here
    
    if(!lpBuff || !lpVPI)
        return;
    
    //Padding spaces
    //One space for the marker character    
    lstrcpy(lpBuff,"  ");       	
    lstrcpy(lpBuff + 2, lpDesc);	
    
    //Add the item to the list  box if it's level is
    //within the viewable levels
    if(nLevel <= lpvp->nLogLevel)
    {
        lpVPI->IsInBox = TRUE;
        if(lpvp->fInversion)
           nret = (int) SendDlgItemMessage(lpvp->lptd->hDlg, IDC_VIEWPORT,
           								   LB_INSERTSTRING, 0, (LPARAM)lpVPI);
        else
           nret = (int) SendDlgItemMessage(lpvp->lptd->hDlg, IDC_VIEWPORT, 
           								   LB_ADDSTRING, 0, (LPARAM)lpVPI);
        
        if(nret == (int)LB_ERRSPACE)
        	MSGINF("Viewport is Out of Memory \n Need To Reset");
   	
    }
    //Scroll if neccessary
    //So how do I scroll ?
    //Detect if new item's rect is outside of viewable listbox
    //If so scroll the list box	.Only do this if we are not
    //inverted !
    if(!lpvp->fInversion)
    {
        int  cItems;
        RECT rcItem, rcList, rcTemp;

        cItems =(int) SendDlgItemMessage(lpvp->lptd->hDlg, IDC_VIEWPORT,
        								 LB_GETCOUNT,      0, 0);
                                         
        GetClientRect(GetDlgItem(lpvp->lptd->hDlg, IDC_VIEWPORT), &rcList );
        
        SendDlgItemMessage(lpvp->lptd->hDlg, IDC_VIEWPORT, LB_GETITEMRECT,  
        				   (WPARAM)(cItems-1), (LPARAM) &rcItem);
        
        //IntersectRect returns the intersection of two rects in
        //rcTemp , in the normal case rcItem should be equal to
        //rcTemp. If they aren't equal, it is time to scroll                   
        IntersectRect(&rcTemp, &rcList, &rcItem);
        
        if(    rcTemp.left   != rcItem.left 
        	|| rcTemp.top    != rcItem.top
            || rcTemp.right  != rcItem.right  
            || rcTemp.bottom != rcItem.bottom)
           
            SendDlgItemMessage(lpvp->lptd->hDlg, IDC_VIEWPORT, LB_SETTOPINDEX,
                               (WPARAM)(max(cItems-2, 0)), (LPARAM)0);
    }

}


/*
 -  XVPReset()
 -
 *  Purpose:
 *      Reset the Viewport
 *      
 */

VOID XVPReset(UINT uID)
{

    FreeVPIList( uID);


}

/*
 -  FreeVPIList()
 -
 *  Purpose:
 *      Free a viewport's item list
 *      
 */

VOID FreeVPIList( UINT uID)
{
    LPVPITEM lpVPI;
    LPVP lpvp;
	
	//All access to the item list is synchronized by a critical section
    //this is so as to avoid erroneous situations where one thread
    //may be attempting to access an item that has been freed by another thread
    //This is an issue for _WIN32  only
     

    lpvp = VPFromID(uID);
	if(!lpvp) return;

#ifdef _WIN32
	if(bMultiThread)
    	EnterCriticalSection(&cs);
#endif

    SendDlgItemMessage(lpvp->lptd->hDlg, IDC_VIEWPORT, LB_RESETCONTENT, 0, 0);
	
	//Iterate thru the least freeing as you go
    while(lpvp->lpItemHead)
    {
        lpVPI = lpvp->lpItemHead;
        lpvp->lpItemHead = lpvp->lpItemHead->next;
        GlobalFreePtr(lpVPI->sz);
        GlobalFreePtr(lpVPI);
    }
	lpvp->lpItemTail = NULL;

#ifdef _WIN32
	if(bMultiThread)
    	LeaveCriticalSection(&cs);
#endif

}


/*
 -  XVPSetLogLevel()
 -
 *  Purpose:
 *      Set the global loglevel
 *      
 */

VOID XVPSetLogLevel(UINT uID, int nLevel)
{
    LPVP lpvp;
    lpvp = VPFromID(uID);
	if(!lpvp) return;

    lpvp->nLogLevel = nLevel;
}

#ifdef _WIN32
/*
 -  ViewPortThread()
 -
 *  Purpose:
 *      This is the thread that manages the Viewport dialog proc.
 *
 *  Parameters:
 *      lptd            - Pointer to a Thread Data object
 *
 *  Returns:
 *      0L
 */

long ViewPortThread(LPVP  lpvp )
{

    dlgproc = (DLGPROC)MakeProcInstance(ViewPortDlgProc, GhInst);
    if((DialogBoxParam(GhInst, "XViewPort", NULL, dlgproc, (LPARAM) lpvp)) == -1)
    {
    	MSGINF("Cannot create Viewport Dialog");
	    SetEvent(lpvp->lptd->hEvent);
        FreeProcInstance((FARPROC)dlgproc);
		ExitThread(1);
    }
    FreeProcInstance((FARPROC)dlgproc);
	
    ExitThread(0);

    return 0L;
}
#endif

/*
 -  ViewPortDlgProc()
 -
 *  Purpose:
 *      The  Viewport dialog procedure.
 *
 */

BOOL CALLBACK ViewPortDlgProc(HWND hDlg, UINT msg, WPARAM wParam, LPARAM lParam)
{
    static int 		pl ;
    LPVP   			lpvp        = NULL;
    static  HBRUSH 	hGrayBrush  = NULL; 
    static  BOOL    fEnded      = FALSE;
    
    //Get the viewport in question
    lpvp = VPFromHwnd(hDlg);
	if(!lpvp) return FALSE;
    
    switch(msg)
    {
        case WM_INITDIALOG:
        {
            HDC   		hDC;
            HWND  		hLB;
            HFONT 		hOldFont = 0;
            RECT        rc;
            //The case for WM_INITDIALOG 
            lpvp             = 	(LPVP)lParam;
            lpvp->lptd->hDlg =   hDlg;

         	//If a last position exists for this port
            //Move the viewport there
            rc = lpvp->rcPos;
            if(!IsRectEmpty(&rc)&& rc.left   >= 0
            					&& rc.top    >= 0
            					&& rc.right  >= 0
            					&& rc.bottom >= 0)
                MoveWindow(hDlg,
                		  lpvp->rcPos.left, 
                		  lpvp->rcPos.top, 
                		  lpvp->rcPos.right  - lpvp->rcPos.left,
                          lpvp->rcPos.bottom - lpvp->rcPos.top, 
                          TRUE);
            else
                XVPPosition(lpvp->uID, pl = TOP_RIGHT);

            //If a saved font exists for this port
            //use it otherwise use the system fixed font

            hLB = GetDlgItem(hDlg, IDC_VIEWPORT);
            hDC = GetDC(hLB);
            if(lpvp->lf.lfHeight == 0)
            {
                lpvp->hCurFont = GetStockObject(ANSI_FIXED_FONT);
                GetObject((HGDIOBJ)lpvp->hCurFont, sizeof(LOGFONT), &lpvp->lf);
            }    
            else
                lpvp->hCurFont = CreateFontIndirect(&lpvp->lf) ;

            SendMessage(hDlg, WM_SETFONT, (WPARAM)lpvp->hCurFont ,FALSE  );
            ReleaseDC(hLB, hDC);
            
            //Enable horizontal scrolling for some arbitrary value
            SendDlgItemMessage(hDlg, IDC_VIEWPORT,  LB_SETHORIZONTALEXTENT,
            				   4096, 0 );
            
            //Set the Toolbar state.
            //We invert the flag so we can use the same code
            //that the user uses when they hide or show the toolbar
            lpvp->bToolbar = !lpvp->bToolbar;
            SendMessage(hDlg, WM_COMMAND, MAKEWPARAM(ID_VIEW_TOOLBAR1,0), 0 );

#ifdef _WIN32
			//If we are threaded, we reset the event here
			//This signals the primary thread to continue
			//since at this point we do have a window handle
			//for the dialog
			if(bMultiThread)
	            SetEvent(lpvp->lptd->hEvent);
#endif
            //Create a light gray brush and 
            hGrayBrush = GetStockObject(LTGRAY_BRUSH);
            lpvp->hLBBrush   = CreateSolidBrush(lpvp->clBkgColor);
           
            return(TRUE);

        }
        case WM_SIZE:
        {

            int 	nWidth 	=  LOWORD(lParam);
            int 	nHeight	=  HIWORD(lParam);
            RECT 	rcView;

			//These are hardcoded dlg dimensions
            //from the rc file. We need to move 
            //the list box accordingly allowing
            //for whether we have a toolbar or not.
            //By the way the toolbar is simply the
            //exposed part of the dialog
            
            if(lpvp->bToolbar)
                SetRect(&rcView, 0, 24, 214, 228);
            else

                SetRect(&rcView, 0, 0,  214, 228);


            MapDialogRect(hDlg, &rcView );
            rcView.right  = nWidth;
            rcView.bottom = nHeight-rcView.top ;

            MoveWindow(GetDlgItem(hDlg, IDC_VIEWPORT),rcView.left, rcView.top,
                                  rcView.right, rcView.bottom, TRUE);
			
			//Save the current position and dimensions
            GetWindowRect(hDlg, &lpvp->rcPos);
            return(FALSE);

        }

        case WM_LBUTTONDBLCLK:
        {
			//Move the viewport to a strategic position
            XVPPosition(lpvp->uID,  (pl % 5 ) + 1);
            pl++;
            return TRUE;
        }
        case WM_CLOSE:
        	//End appropriately . Remember we could be
            //a modal dlg or a modeless dlg
            DeleteObject(lpvp->hCurFont);
        
			if(bMultiThread)
			{
        	#ifdef _WIN32
            	EndDialog(hDlg, TRUE);
				fEnded = TRUE;
            	return(TRUE);
			
        	#endif
			}
			
            DestroyWindow(hDlg);
            return TRUE;

#ifdef _WIN32

        case WM_CTLCOLORDLG:
        	//Return a gray brush for the dialog bkg
            return (BOOL)hGrayBrush ;


        case WM_CTLCOLORLISTBOX:
        	//Change the list box text and bkg colors
            SetTextColor((HDC)wParam, lpvp->clTextColor);
            SetBkColor((HDC)wParam, lpvp->clBkgColor);
            return (BOOL)lpvp->hLBBrush;
#else

        case WM_CTLCOLOR:
        	//Change the list box text and bkg colors
			if (HIWORD(lParam) == CTLCOLOR_LISTBOX)
            {
                SetTextColor((HDC)wParam, lpvp->clTextColor);
                SetBkColor((HDC)wParam, lpvp->clBkgColor);
                return (BOOL)lpvp->hLBBrush;
            }
            return (HBRUSH) NULL;

#endif
        case WM_MEASUREITEM:
        {

            TEXTMETRIC tm;
            HDC   hDC;
            HWND  hLB;
            HFONT hOldFont = 0;
            LPMEASUREITEMSTRUCT lpmis;
			
			if(fEnded)
    			return TRUE;

			// get the DC for the listbox
            lpmis = (LPMEASUREITEMSTRUCT)lParam;
            hLB = GetDlgItem(hDlg, lpmis->CtlID);
            hDC = GetDC(hLB);
            
            

			//Measure the item in terms of the current font
            hOldFont = SelectObject(hDC, lpvp->hCurFont);

            GetTextMetrics(hDC, &tm);

			//add an extra four pixels
            lpmis->itemHeight = tm.tmHeight + 4;
            if (hOldFont)
                SelectObject(hDC, hOldFont);

            ReleaseDC(hLB, hDC);
            return (TRUE);
        }

        case WM_DRAWITEM:
        {
            LPRECT    			lpRCT;
            HFONT     			hOldFont;
            LPDRAWITEMSTRUCT  	lpdis;
            LPVPITEM  			lpItem ;
            int       			xIndent ;

    		if(fEnded)
    			return TRUE;    

		
		    // Get pointer to the DRAWITEMSTRUCT
            lpdis  = (LPDRAWITEMSTRUCT)lParam;
            if( lpdis->itemID == -1)
                return TRUE;
            
            lpItem = (LPVPITEM)(lpdis->itemData);
            if( !lpItem )
                return TRUE;

            lpRCT  =  &lpdis->rcItem;
            xIndent = lpItem->nLevel * (8  * lpvp->nTabCnt);

            //Set the background and text color
            //based on the item's selection state.
            if (lpdis->itemState & ODS_SELECTED)
            {
               SetBkColor(lpdis->hDC, GetSysColor(COLOR_HIGHLIGHT));
               SetTextColor(lpdis->hDC, GetSysColor(COLOR_HIGHLIGHTTEXT));
            }
            else
            {
               //SetBkColor(lpdis->hDC, GetSysColor(COLOR_WINDOW));
               SetBkColor(lpdis->hDC, lpvp->clBkgColor);
               if(lpItem->nType & FAIL )
               		//Bright red for failure
                     SetTextColor(lpdis->hDC, RGB(192, 0, 0)); 
               else
               if( lpItem->nType & PASS )
               		//Bright  green for success
                    SetTextColor(lpdis->hDC, RGB(0, 156, 0 )); 
               else
                    SetTextColor(lpdis->hDC, lpvp->clTextColor);
            }

            hOldFont = SelectObject(lpdis->hDC, lpvp->hCurFont);
            
            //Determine if this is a container or not
            if(lpItem->next && lpItem->nLevel >= lpItem->next->nLevel)
            {
                //The TAIL case
                if(lpItem->nType & TAIL)
                {
                    lpItem->sz[0] = '^';
                }
                else //The normal case
                    lpItem->sz[0] = ' ';
            }
            else
                if(!lpItem->next)
                    lpItem->sz[0] =' ';

                else
                    if(lpItem->IsOpen)
                        lpItem->sz[0] = '-';
                    else
                        if(lpItem->nType & TAIL)
                            lpItem->sz[0] = '^';
                        else
                            lpItem->sz[0] = '+';
			//Draw the text
           	ExtTextOut( lpdis->hDC, 
           				lpRCT->left + 4 + xIndent , 
           				lpRCT->top + 2, 
           				ETO_CLIPPED | ETO_OPAQUE, lpRCT,
                        lpItem->sz,
                        lstrlen(lpItem->sz), NULL);

            if (!lpItem->nType & HEAD)
                SelectObject(lpdis->hDC, hOldFont);
		

            return (TRUE);

        }
        case WM_COMMAND:
        {
            switch (LOWORD(wParam))
            {
                case IDC_VIEWPORT:
                {
                	//List box notification
                    //We only process double mouse clicks
                    
                    int idx;
                    int inext;
                    LPVPITEM lpItemP = NULL;
                    LPVPITEM lpItemK = NULL;
#ifdef _WIN32
                    if(HIWORD(wParam) != LBN_DBLCLK)
#else
                    if(HIWORD(lParam) != LBN_DBLCLK)
#endif
                        return TRUE;

					idx = (int)SendDlgItemMessage(hDlg,IDC_VIEWPORT, 
												  LB_GETCURSEL, 0 , 0 );
                    if(lpvp->fInversion)
                        inext = max(0, idx - 1);
                    else
                        inext = idx + 1;

					//Get the current item the one highlighted in the listbox
                    lpItemP = (LPVPITEM)SendDlgItemMessage(hDlg, IDC_VIEWPORT, 
                    									   LB_GETITEMDATA, 
                    									   idx, 0);
                    //Get the next item in the list box                                       
                    lpItemK = (LPVPITEM)SendDlgItemMessage(hDlg, IDC_VIEWPORT, 
                    									   LB_GETITEMDATA, 
                    									   inext, 0);

                    if( lpItemK == (LPVPITEM)LB_ERR)
                    {
                            if(!lpItemP->next)
                                return TRUE;
                    }
                    
                    //Prepare for redrawing
                    SendDlgItemMessage(hDlg, IDC_VIEWPORT, 
                    				   WM_SETREDRAW, FALSE, 0);

                    //If we are drawing a tail the behavior is different
                    //We need the predecessor within the listbox
                    if(lpItemP->nType & TAIL )
                        goto DrawTail;

                    //If the item is open we now close it
                    if(lpItemP->IsOpen)
                    {

                        lpItemP->IsOpen = FALSE;
                        while(lpItemK != (LPVPITEM)LB_ERR 
                                      && lpItemK->nLevel > lpItemP->nLevel)
                        {
                            lpItemK->IsOpen  = FALSE;
                            lpItemK->IsInBox = FALSE;
                            if(lpvp->fInversion)
                            {
                            	//If we are inverted the kids are above us
                                SendDlgItemMessage(hDlg, IDC_VIEWPORT,
                                				   LB_DELETESTRING, 
                                				   inext-- , 0 );
                                                   
                                lpItemK = 
                                (LPVPITEM)SendDlgItemMessage(hDlg, 
                                                             IDC_VIEWPORT,
                                                             LB_GETITEMDATA,
                                                             inext, 0);

                            }
                            else
                            {
                            	//If we are not inverted the kids are below us
                                SendDlgItemMessage(hDlg, IDC_VIEWPORT, 
                                				   LB_DELETESTRING, 
                                				   inext, 0 );
                                                   
                                lpItemK = 
                                (LPVPITEM)SendDlgItemMessage(hDlg,
                                							 IDC_VIEWPORT,
                                							 LB_GETITEMDATA, 
                                							 inext, 0);

                            }
                        }

                    }

                    //If the item is closed we now open it
                    else
                    {

                        BOOL     fNextExist  = FALSE;
                        LPVPITEM lpItemTemp  = NULL;
                        int      nMinLevel  ;

                        lpItemP->IsOpen = TRUE;
                        lpItemK = lpItemP->next;

                        //Find minimum level > current level
                        lpItemTemp = lpItemK;
                        nMinLevel  = lpItemK->nLevel;
                        while(lpItemTemp && lpItemTemp->nLevel > lpItemP->nLevel)
                        {
                            if(lpItemTemp->nLevel < nMinLevel)
                                nMinLevel = lpItemTemp->nLevel;
                            lpItemTemp = lpItemTemp->next;
                        }

                        while(lpItemK && lpItemK->nLevel > lpItemP->nLevel )
                        {

                            //Open all items with minimum level
                            if(lpItemK->nLevel == nMinLevel )
                            {
                                lpItemK->IsInBox = TRUE;
                                if(lpvp->fInversion)
                                    SendDlgItemMessage(hDlg, IDC_VIEWPORT, 
                                    				   LB_INSERTSTRING,  
                                    				   idx, (LPARAM) lpItemK );
                                else
                                    SendDlgItemMessage(hDlg, IDC_VIEWPORT, 
                                                       LB_INSERTSTRING, ++idx, 
                                                       (LPARAM) lpItemK );
                            }
                            lpItemK = lpItemK->next;
                        }

                    }
					SendDlgItemMessage(hDlg, IDC_VIEWPORT, WM_SETREDRAW, TRUE, 0);
					return TRUE;

             DrawTail:
                    {
                        LPVPITEM lpItemCur = lpItemP;;

                        if(lpItemCur->IsOpen)
                        {
                            //We have to go backward in the list box until we find the
                            //the next lower level element
                            while(TRUE)
                            {
                                if(lpvp->fInversion)
                                    lpItemP = 
                                    (LPVPITEM)SendDlgItemMessage(hDlg, 
                                    							 IDC_VIEWPORT, 
                                                                 LB_GETITEMDATA, 
                                                                 ++idx, 0);
                                else
                                    lpItemP =
                                    (LPVPITEM)SendDlgItemMessage(hDlg, 
                                    							 IDC_VIEWPORT, 
                                    							 LB_GETITEMDATA,
                                    							 --idx, 0);

                                if(lpItemP->nLevel <= lpItemCur->nLevel)
                                    break;
                            }

                            if(lpvp->fInversion)

                                inext = max(0, idx - 1);
                            else
                                inext = idx + 1;

                            lpItemK = 
                            (LPVPITEM)SendDlgItemMessage(hDlg, 
                            							 IDC_VIEWPORT,
                            							 LB_GETITEMDATA, 
                            							 inext, 0);
                            lpItemCur->IsOpen = FALSE;
                            while(lpItemK != (LPVPITEM)LB_ERR 
                            	  && lpItemK->nLevel > lpItemCur->nLevel)
                            {
                                lpItemK->IsOpen  = FALSE;
                                lpItemK->IsInBox = FALSE;
                                if(lpvp->fInversion)
                                {
                                    SendDlgItemMessage(hDlg, IDC_VIEWPORT, 
                                                       LB_DELETESTRING, 
                                                       inext-- , 0 );
                                                       
                                    lpItemK =(LPVPITEM)
                                    SendDlgItemMessage(hDlg, 
                                    				   IDC_VIEWPORT,
                                    				   LB_GETITEMDATA, 
                                    				   inext, 0);

                                }
                                else
                                {
                                    SendDlgItemMessage(hDlg, 
                                    				   IDC_VIEWPORT, 
                                    				   LB_DELETESTRING, 
                                    				   inext, 0 );
                                    lpItemK = (LPVPITEM)
                                    SendDlgItemMessage(hDlg, 
                                    				   IDC_VIEWPORT, 
                                    				   LB_GETITEMDATA, 
                                    				   inext, 0);

                                }
                            }

                        }
                        else
                        {
                            if(lpvp->fInversion)
                            {
                                idx++;
                                inext = max(0, idx - 1);
                            }
                            else
                            {   idx--;
                                inext = idx + 1;
                            }
                            lpItemP = 
                            (LPVPITEM)SendDlgItemMessage(hDlg, 
                            						     IDC_VIEWPORT, 
                            						     LB_GETITEMDATA, 
                            						     idx , 0);
                                                         
                            lpItemK = 
                            (LPVPITEM)SendDlgItemMessage(hDlg, 
                            							 IDC_VIEWPORT, 
                            							 LB_GETITEMDATA, 
                            							 inext, 0);

                            lpItemCur->IsOpen = TRUE;
                            lpItemK = lpItemP->next;
                            while(lpItemK && 
                                  lpItemK->nLevel > lpItemCur->nLevel )
                            {

                                if(lpItemK->nLevel == lpItemCur->nLevel + 1)
                                {
                                    lpItemK->IsInBox = TRUE;
                                    if(lpvp->fInversion)
                                        SendDlgItemMessage(hDlg, 
                                        				   IDC_VIEWPORT, 
                                        				   LB_INSERTSTRING,  
                                        				   idx, (LPARAM) lpItemK );
                                    else
                                        SendDlgItemMessage(hDlg, IDC_VIEWPORT, 
                                                           LB_INSERTSTRING, ++idx, 
                                                           (LPARAM) lpItemK );
                                }
                                lpItemK = lpItemK->next;
                            }


                        }
                        SendDlgItemMessage(hDlg, IDC_VIEWPORT,
                                           WM_SETREDRAW, TRUE, 0);
						return TRUE;
                    }
                    SendDlgItemMessage(hDlg, IDC_VIEWPORT,
                    				   WM_SETREDRAW, TRUE, 0);
					return TRUE;

                }

                case ID_FILE_OPEN1:
                {

                    OPENFILENAME ofn;
                    int nret        = 0;
                    char szFile[MAX_PATH];
                    char szFileTitle[MAX_PATH];
                    int i;

                    szFile[0]      = '\0';
                    szFileTitle[0] =  '\0';


                    ofn.lStructSize         = sizeof(OPENFILENAME);
                    ofn.hwndOwner           = lpvp->lptd->hDlg;
                    ofn.lpstrFilter         = (LPSTR) "XVP Files (*.xvp)\0 \
*.xvp\0All Files (*.*)\0*.*\0";
                    ofn.lpstrCustomFilter   = NULL;
                    ofn.nMaxCustFilter      = 0;
                    ofn.nFilterIndex        = 0;
                    ofn.lpstrFile           = (LPSTR)szFile;
                    ofn.nMaxFile            = 256;
                    ofn.lpstrInitialDir     = NULL;
                    ofn.lpstrTitle          = "XVP Open File";
                    ofn.Flags               = OFN_HIDEREADONLY|
                    						  OFN_FILEMUSTEXIST;
                    ofn.lpstrDefExt         = NULL;
                    ofn.lpfnHook            = NULL;
                    ofn.lCustData           = 0L;
                    ofn.lpTemplateName      = NULL;
                    ofn.nFileExtension      = 0;
                    ofn.lpstrFileTitle      = szFileTitle;
                    ofn.nMaxFileTitle       = MAX_PATH;

                    if(!GetOpenFileName(&ofn))
                        return TRUE;

                    hNormalCursor = SetCursor(hWaitCursor);

                    // If the result is not an empty string
                    // Open the file and read
                    if (*szFile)
                    {
                        HFILE    hFile;
                        LPVPITEM lpItem = NULL;

                        lstrcpy(lpvp->szFile, szFileTitle);

                        //If we are not in Append mode we need to reset
                        if(!lpvp->bAppendMode)
                            XVPReset(lpvp->uID);

                        //First open the file
                        hFile = _lopen(szFile, OF_READ);
                        if(hFile == HFILE_ERROR)
                            return TRUE;

						SendDlgItemMessage(hDlg, IDC_VIEWPORT, 
										   WM_SETREDRAW, FALSE, 0);

                        //Read from file
                        while(lpItem = ItemFromFile(hFile))
                        {
                            //Append the new item to the list;
                            if(!lpvp->lpItemHead)
                            {
                                lpvp->lpItemHead = lpItem;
                                lpvp->lpItemTail = lpvp->lpItemHead;
                            }
                            else
                            {
                                lpvp->lpItemTail->next = lpItem;
                                lpvp->lpItemTail       = lpItem;
                            }

                            if(lpItem->IsInBox)
                            {
                                if(lpvp->fInversion)
                                   nret = 
                                   (int) SendDlgItemMessage(lpvp->lptd->hDlg,
                                                IDC_VIEWPORT, LB_INSERTSTRING,
                                                0, (LPARAM)lpItem);
                                else
                                   nret =
                                   (int) SendDlgItemMessage(lpvp->lptd->hDlg,
                                                IDC_VIEWPORT, LB_ADDSTRING, 0,
                                                (LPARAM)lpItem);

                                if(nret == (int)LB_ERRSPACE)
                                    MSGINF("Viewport is Out of\
                                    Memory \n Need To Reset");

                            }

                        }
                        SendDlgItemMessage(hDlg, IDC_VIEWPORT, 
                        				   WM_SETREDRAW, TRUE, 0);
                        _lclose(hFile);
                    
                    	//Set the Viewport Title
                    	GetWindowText(hDlg, szFile, MAX_PATH);
                    	for(i = lstrlen(szFile); i >= 0; i-- )
                    	    if(szFile[i] == '-')
                    	    {
                    	      szFile[i+1] = '\0';
                    	      lstrcat(szFile, " ");
                    	      lstrcat(szFile, szFileTitle);
                    	      XVPSetTitle(lpvp->uID,szFile);
                    	      lstrcpy(lpvp->szTitle, szFile);
                    	      SetCursor(hNormalCursor);
                    	      return TRUE;
                    	    }

                    	lstrcat(szFile, " - ");
                    	lstrcat(szFile, szFileTitle);
                    	XVPSetTitle(lpvp->uID, szFile);
                    	lstrcpy(lpvp->szTitle, szFile);
                    	SetCursor(hNormalCursor);
                    	return TRUE;
                    }
                }
                case ID_FILE_SAVE1:
                {
                    OPENFILENAME ofn;
                    char szFile[MAX_PATH];
                    char szFileTitle[MAX_PATH];

                    szFile[0]      = '\0';
                    szFileTitle[0] =  '\0';
                    lstrcpy(szFile, lpvp->szFile);
                    ofn.lStructSize         = sizeof(OPENFILENAME);
                    ofn.hwndOwner           = lpvp->lptd->hDlg;
                    ofn.lpstrFilter         = (LPSTR) "XVP Files (*.xvp)\0\
*.xvp\0All Files (*.*)\0*.*\0";
                    ofn.lpstrCustomFilter   = NULL;
                    ofn.nMaxCustFilter      = 0;
                    ofn.nFilterIndex        = 0;
                    ofn.lpstrFile           = (LPSTR)szFile;
                    ofn.nMaxFile            = MAX_PATH;
                    ofn.lpstrInitialDir     = NULL;
                    ofn.lpstrTitle          = "XVP Save File";
                    ofn.Flags               = OFN_OVERWRITEPROMPT |
                    						  OFN_HIDEREADONLY;
                    ofn.lpstrDefExt         = NULL;
                    ofn.lpfnHook            = NULL;
                    ofn.lCustData           = 0L;
                    ofn.lpTemplateName      = NULL;
                    ofn.nFileExtension      = 0;
                    ofn.lpstrFileTitle      = szFileTitle;
                    ofn.nMaxFileTitle       = MAX_PATH;



                    if(!GetSaveFileName(&ofn))
                        return TRUE;

                    hNormalCursor = SetCursor(hWaitCursor);
                    // If the result is not the empty string
                    // Open the file and save
                    if (*szFile)
                    {
                        HFILE    hFile;
                        LPVPITEM lpItem = NULL;
                        LPVPITEM lpT;

                        //First open the file
                        hFile = _lcreat(szFile, 0);
                        if(hFile == HFILE_ERROR)
                         	return TRUE;

                        //Put item to file
                        lpT = lpvp->lpItemHead;
                        while(lpT)
                        {
                             lpItem = lpT;
                             lpT = lpT->next;
                             if(lpvp->nSaveLevel != -1 &&
                                lpItem->nLevel > lpvp->nSaveLevel )
                                continue;

                             if(! FItemToFile(hFile, lpItem))
                                break;
                        }
                        _lclose(hFile);
                    }
                    SetCursor(hNormalCursor);
                        return TRUE;
                    
                }
                case ID_FILE_EXIT:
                    PostMessage(hDlg, WM_CLOSE, 0, 0);
                        return TRUE;
                    
                case ID_OPTIONS_OPTIONS:
                {
                    DLGPROC dlgproc;

                    dlgproc = 
                    (DLGPROC)MakeProcInstance(OptionsDlgProc,	GhInst);
                                                        
                    DialogBoxParam(GhInst, "Options", 
                    			   hDlg, dlgproc, (LPARAM) lpvp);
                                   
                    FreeProcInstance((FARPROC)dlgproc);
                    return TRUE;
                    
                }
                case ID_OPTIONS_RESET:
                {
                    hNormalCursor = SetCursor(hWaitCursor);
                    XVPReset(lpvp->uID);
                    SetCursor(hNormalCursor);
                    return TRUE;
                    
                }
                case ID_OPTIONS_FONT:
                {

                    HFONT       hNewFont;
                    CHOOSEFONT  cf;
                    HWND        hLB = GetDlgItem(hDlg, IDC_VIEWPORT);
                    LPVPITEM    lpItem;
                    int         nret = 0;

                    cf.lStructSize      = sizeof (CHOOSEFONT);
                    cf.hwndOwner        = hDlg;
                    cf.hDC              = 0L;
                    cf.lpLogFont        = &lpvp->lf;
                    cf.iPointSize       = 0;
                    cf.Flags            = CF_INITTOLOGFONTSTRUCT| 
                    					  CF_SCREENFONTS|
                                          CF_EFFECTS;
                    cf.rgbColors        = lpvp->clTextColor;
                    cf.lCustData        = 0L;
                    cf.lpfnHook         = NULL;
                    cf.lpTemplateName   = NULL;
                    cf.hInstance        = 0;
                    cf.lpszStyle        = NULL;
                    cf.nFontType        = 0;
                    cf.nSizeMin         = 0;
                    cf.nSizeMax         = 0;


                    if(! ChooseFont ((CHOOSEFONT FAR *)&cf) )
                        return TRUE;
                        

                    hNormalCursor = SetCursor(hWaitCursor);
                    DeleteObject(lpvp->hCurFont);
                    hNewFont = CreateFontIndirect (&lpvp->lf) ;
                    lpvp->hCurFont = hNewFont;
                    SendMessage (hLB, WM_SETFONT, (WPARAM)hNewFont, TRUE);
                    lpvp->clTextColor = cf.rgbColors;

                    //Reset the display so as to recalculate
                    SendMessage(hLB, LB_RESETCONTENT, 0 , 0 );

                    //Restore the display
                    SendDlgItemMessage(hDlg, IDC_VIEWPORT, 
                    				   WM_SETREDRAW, FALSE, 0);
                    lpItem = lpvp->lpItemHead;
                    while(lpItem)
                    {
                        if(lpItem->IsInBox)
                        {
                            if(lpvp->fInversion)
                                nret = 
                                (int) SendDlgItemMessage(lpvp->lptd->hDlg,
                                                IDC_VIEWPORT, LB_INSERTSTRING,
                                                0, (LPARAM)lpItem);
                            else
                                nret = 
                                (int) SendDlgItemMessage(lpvp->lptd->hDlg,
                                                IDC_VIEWPORT, LB_ADDSTRING, 0,
                                                (LPARAM)lpItem);

                            if(nret == (int)LB_ERRSPACE)
                                MSGINF("Viewport is Out of\
                                Memory \n Need To Reset");
                        }
                        lpItem = lpItem->next;

                    }

					SendDlgItemMessage(hDlg, IDC_VIEWPORT, 
									   WM_SETREDRAW, TRUE, 0);
                                       
                    SetCursor(hNormalCursor);
                    return TRUE;
                    

                }
                case ID_OPTIONS_BKGCOLOR:
                {
                    CHOOSECOLOR cc;
                    COLORREF    aclrCust[16];
                    int i;
                    LOGBRUSH lb;
                    HWND hLB = GetDlgItem(hDlg, IDC_VIEWPORT);

                    //Set the  custom controls to white
                    for( i=0; i<16; aclrCust[i++] = RGB(255, 255, 255));

                    memset(&cc, 0, sizeof(CHOOSECOLOR));
                    cc.lStructSize   =  sizeof (CHOOSECOLOR);
                    cc.hwndOwner     =  hDlg;
                    cc.Flags         =  CC_PREVENTFULLOPEN;
                    cc.rgbResult     =  lpvp->clBkgColor;
                    cc.lpCustColors  =  aclrCust;

                    if(! ChooseColor (&cc) )
                        return TRUE;
                        
                    lpvp->clBkgColor =  cc.rgbResult;
                    hGrayBrush		 =  GetStockObject(LTGRAY_BRUSH);
                    lb.lbStyle       =  BS_SOLID;
                    lb.lbColor       =  lpvp->clBkgColor;
                    lb.lbHatch       =  0;
                    
                    DeleteObject(lpvp->hLBBrush);
                    lpvp->hLBBrush   = CreateBrushIndirect(&lb);
                    InvalidateRect( hLB ,NULL, TRUE);

                    return TRUE;
                                    }
                case ID_OPTIONS_INVERT:
                {
                    XVPInvert(lpvp->uID);
                    return TRUE;
                    
                }
                case ID_OPTIONS_POSITION_TOPLEFT:
                {
                    XVPPosition(lpvp->uID, TOP_LEFT);
                    return TRUE;
                    
                }
                case ID_OPTIONS_POSITION_TOPRIGHT:
                {
                    XVPPosition(lpvp->uID, TOP_RIGHT);
                    return TRUE;
                    
                }
                case ID_OPTIONS_POSITION_CENTER:
                {
                    XVPPosition(lpvp->uID, CENTER);
                    return TRUE;

                }
                case ID_OPTIONS_POSITION_BOTTOMLEFT:
                {
                    XVPPosition(lpvp->uID, BOTTOM_LEFT);
                    return TRUE;

                }
                case ID_OPTIONS_POSITION_BOTTOMRIGHT:
                {
                    XVPPosition(lpvp->uID,BOTTOM_RIGHT);
                    return TRUE;

                }
                case ID_VIEW_TOOLBAR1:
                {
                    HMENU hMenu  = GetMenu(hDlg);
                    int   nStatus;
                    RECT rcFrame;
                    LPVP lpvp = VPFromHwnd(hDlg);
					if(!lpvp) return TRUE;


                    GetClientRect(hDlg, &rcFrame);
                    
                    lpvp->bToolbar = !lpvp->bToolbar;
                    
                    CheckMenuItem(hMenu, ID_VIEW_TOOLBAR1, 
                    			 (lpvp->bToolbar ? 
                    			 MF_CHECKED : MF_UNCHECKED ) );
                                 
                    if(!lpvp->bToolbar)
                        nStatus = SW_HIDE;
                    else
                        nStatus = SW_SHOW;

                    ShowWindow(GetDlgItem(hDlg,ID_FILE_OPEN1), nStatus);
                    ShowWindow(GetDlgItem(hDlg,ID_FILE_SAVE1), nStatus);
                    ShowWindow(GetDlgItem(hDlg,ID_OPTIONS_RESET), nStatus);
                    ShowWindow(GetDlgItem(hDlg,ID_OPTIONS_INVERT), nStatus);

                    SendMessage(hDlg, WM_SIZE, (WPARAM)SIZE_RESTORED,
                                MAKELPARAM(rcFrame.right-rcFrame.left,
                                           rcFrame.bottom-rcFrame.top));
                    return TRUE;



                }
                case ID_VIEW_STATUSBAR1:
                    NYI;
                    return TRUE;

                case ID_HELP_ABOUT:
                {
                    DLGPROC dlgproc;
                    dlgproc = (DLGPROC)MakeProcInstance(AboutDlgProc, GhInst);
                    DialogBox(GhInst, "About", hDlg, dlgproc);
                    FreeProcInstance((FARPROC)dlgproc);
                    return TRUE;


                }
                default:
					return FALSE;
            }
			
        }
        default:
            return FALSE;

    }
    

}

/*
 -  OptionsDlgProc()
 -
 *  Purpose:
 *      The  Options dialog procedure.
 *
 */

BOOL CALLBACK OptionsDlgProc(HWND hDlg, UINT msg, WPARAM wParam, LPARAM lParam)
{

    static LPVP lpvp;

    switch(msg)
    {
        case WM_INITDIALOG:
        {
            lpvp = (LPVP)lParam;
            CheckDlgButton(hDlg, IDC_RADIOALLLEVELS, TRUE );
            CheckDlgButton(hDlg, IDC_CHECKAPPENDMODE, lpvp->bAppendMode);
            SetDlgItemInt(hDlg,  IDC_EDITLEVEL,   lpvp->nLogLevel, FALSE);
            SetDlgItemInt(hDlg,  IDC_EDITTABS,    lpvp->nTabCnt, FALSE);
            if(lpvp->nSaveLevel > -1)
                SetDlgItemInt(hDlg,  IDC_EDITSAVELEVEL, 
                			  lpvp->nSaveLevel, FALSE );
            return TRUE;

        }
        case WM_COMMAND:
            if(LOWORD(wParam) != IDOK)
                return TRUE;


        case WM_CLOSE:
        {
            BOOL bTranslated;
            if(IsDlgButtonChecked(hDlg, IDC_RADIOALLLEVELS))
                lpvp->nSaveLevel = -1 ;
            else
                lpvp->nSaveLevel = GetDlgItemInt(hDlg, IDC_EDITSAVELEVEL, 
                								 &bTranslated, FALSE);

            lpvp->nLogLevel = GetDlgItemInt(hDlg, IDC_EDITLEVEL,   
            							   &bTranslated, FALSE);
            lpvp->nTabCnt   = GetDlgItemInt(hDlg, IDC_EDITTABS,    
            							   &bTranslated, FALSE);
            lpvp->bAppendMode = IsDlgButtonChecked(hDlg, IDC_CHECKAPPENDMODE);
            EndDialog(hDlg, TRUE);
            return(TRUE);
        }
        default:
            return FALSE;
	}
}

/*
 -  AboutDlgProc()
 -
 *  Purpose:
 *      The  About dialog procedure.
 *
 */

BOOL CALLBACK AboutDlgProc(HWND hDlg, UINT msg, WPARAM wParam, LPARAM lParam)
{


    switch(msg)
    {
        case WM_INITDIALOG:
        {

            return TRUE;
        }
        case WM_COMMAND:
            if(LOWORD(wParam) != IDOK)
               return TRUE;

        case WM_CLOSE:
            EndDialog(hDlg, TRUE);
            return(TRUE);

        default:
            return FALSE;
    }
    
}

/*
 - XVPPosition
 -
 * Place a Dialog in a requested position on the Desktop
 *
 */

void  XVPPosition (UINT uID, int iPlacement)
{
    LPVP lpvp ;
    HWND hDlg ;
    HWND hDt  ;
    WINDOWPLACEMENT wpDt  ;
    WINDOWPLACEMENT wpDlg ;
	RECT Dtrc;
    RECT Dlgrc;
    
#ifdef WIN16
    RECT rcDt;
#endif


	lpvp = VPFromID( uID);
	if(!lpvp) return;

    hDlg = lpvp->lptd->hDlg;
    hDt  = GetDesktopWindow ();
    wpDt.length = wpDlg.length = sizeof(WINDOWPLACEMENT);


#ifdef WIN16
    GetWindowRect(hDt, &rcDt);
    wpDt.rcNormalPosition = rcDt;
#else
    GetWindowPlacement(hDt, &wpDt);
#endif

    GetWindowPlacement(hDlg, &wpDlg);

	Dtrc = wpDt.rcNormalPosition;
    Dlgrc= wpDlg.rcNormalPosition;

	//Validate the Desktop window placement Rect
    if(	Dtrc.left < 0 || Dtrc.top    < 0
    				  || Dtrc.right	 < 0
                      || Dtrc.bottom < 0 )
		return;                      

    switch( iPlacement)
    {

    case TOP_LEFT:
        MoveWindow(hDlg,
           Dtrc.left,
           Dtrc.top,
           Dlgrc.right   - Dlgrc.left,
           Dlgrc.bottom  - Dlgrc.top,
           TRUE);

    break;


    case TOP_RIGHT:
        MoveWindow(hDlg,
          Dtrc.right   - Dtrc.left - (Dlgrc.right  - Dlgrc.left),
          Dtrc.top,
          Dlgrc.right   - Dlgrc.left,
          Dlgrc.bottom  - Dlgrc.top,
          TRUE);

        break;


    case CENTER:
        MoveWindow(hDlg,
          (Dtrc.right   - Dtrc.left -  (Dlgrc.right  - Dlgrc.left))/2,
          (Dtrc.bottom  - Dtrc.top  -  (Dlgrc.bottom - Dlgrc.top))/2,
          Dlgrc.right   - Dlgrc.left,
          Dlgrc.bottom  - Dlgrc.top,
          TRUE);

        break;

    case BOTTOM_LEFT:
        MoveWindow(hDlg,
          Dtrc.left,
          Dtrc.bottom   - Dtrc.top - (Dlgrc.bottom - Dlgrc.top),
          Dlgrc.right   - Dlgrc.left,
          Dlgrc.bottom  - Dlgrc.top,
          TRUE);

        break;

    case BOTTOM_RIGHT:
        MoveWindow(hDlg,
          Dtrc.right    - Dtrc.left - (Dlgrc.right  - Dlgrc.left),
          Dtrc.bottom   - Dtrc.top  - (Dlgrc.bottom - Dlgrc.top),
          Dlgrc.right   - Dlgrc.left,
          Dlgrc.bottom  - Dlgrc.top,
          TRUE);

        break;


    default:
        break;
    }

}

/*
 - FItemToFile
 -
 * 	Purpose
 *		   Write an item to file 
 */

BOOL FItemToFile(HFILE hfile, LPVPITEM lpitem )
{
    UINT  res,stlen;
    char  lpT1[6],lpT2[6],lpT3[6],lpT4[6];
    LPSTR sz,lpT;

    /* if item is NULL return FALSE */

    if(!lpitem)
       return FALSE;

    /*
     otherwise counnt the number of byte to write
    */

    sz     = NULL;
    lpT    = itoa(lpitem->nType, lpT1, 10);
    stlen  = lstrlen(lpT) + 1;

    lpT    = itoa(lpitem->nLevel, lpT2, 10);
    stlen += lstrlen(lpT) + 1;

    lpT    = itoa(lpitem->IsOpen, lpT3, 10);
    stlen += lstrlen(lpT) + 1;

    lpT    = itoa(lpitem->IsInBox, lpT4, 10);
    stlen += lstrlen(lpT) + lstrlen(lpitem->sz) + 2;

    /* allocate the string to write , make a single
       string of all the strings to write */

    sz     = (LPSTR) GlobalAllocPtr(GHND,stlen+1);
    sz     = lstrcat(sz,lpT1);
    sz     = lstrcat(sz,",");
    sz     = lstrcat(sz,lpT2);
    sz     = lstrcat(sz,",");
    sz     = lstrcat(sz,lpT3);
    sz     = lstrcat(sz,",");
    sz     = lstrcat(sz,lpT4);
    sz     = lstrcat(sz,":\t");
    sz     = lstrcat(sz,lpitem->sz);

    /* write the concatenated string */

    res    = _lwrite(hfile,sz,lstrlen(sz));
    if(res == HFILE_ERROR) return 0;

    /* add carriage return and endofline chars
       after the string in the file  */

    res    = _lwrite(hfile,"\r\n",2);
    if(res == HFILE_ERROR) return 0;
    return 1;
}

/*
 - VPFromID
 -
 * 	Purpose
 *		   Identify a Viewport structure
 *		   using an ID
 */
 
LPVP VPFromID( UINT uID)
{
    LPVP lpT = VPHead;

    while(lpT)
    {
        if(lpT->uID == uID)
            return lpT;
        lpT = lpT->next;
    }


}

/*
 - VPFromHwnd
 -
 * 	Purpose
 *		   Identify a Viewport structure
 *		   using an HWND
 */

LPVP VPFromHwnd( HWND hDlg)
{
    LPVP lpT = NULL;
    lpT = VPHead;

    while(lpT)
    {
        if(lpT->lptd->hDlg == hDlg)
            return lpT;
        lpT = lpT->next;
    }


}

/*
 - ItemFromFile
 -
 * 	Purpose
 *		   Read an item from file 
 */


LPVPITEM ItemFromFile(HFILE hfile)
{
 
	UINT     cbread; 
 	LONG     count,icount,pos,chead;
 	LPSTR    str,lpT;
 	LPVPITEM item;
 	char     szNum[6];

 	count      = 0;
 	icount     = 0;
 	str        = (LPSTR) GlobalAllocPtr(GHND,512);
 	pos        = _llseek(hfile,0L,1);
 	item       = (LPVPITEM) GlobalAllocPtr(GHND,sizeof(struct _vpitem));
 	item->next = NULL;
 	cbread     = _lread(hfile,str,512);
 	if (cbread != 0) {

 	 /*
 	    Strip the initial escape chars if any
 	 */

 	 while(icount < (long) cbread) {
 	  if(((*str) == '\n') || ((*str) == '\r')) {
 	   ++str;
 	   ++icount;
 	  }
 	  else break;
 	 }

 	 /* If end of file found in the process of stripping
 	    escape chars return NULL */

 	 if(icount == (long) cbread) {
 	  GlobalFreePtr(str);
 	  return NULL;
 	 }

 	 /* Get  nType,nLevel,IsOpen,IsInBox from the
 	    buffer str */

 	 lstrcpy(szNum,strtok(str,","));
 	 chead         = lstrlen(szNum) + 1;
 	 item->nType   = atoi(szNum);
 	 lstrcpy(szNum,strtok(NULL,","));
 	 chead        += lstrlen(szNum) + 1;
 	 item->nLevel  = atoi(szNum);
 	 lstrcpy(szNum,strtok(NULL,","));
 	 chead        += lstrlen(szNum) + 1;
 	 item->IsOpen  = atoi(szNum);
 	 lstrcpy(szNum,strtok(NULL,":"));
 	 chead        += lstrlen(szNum) + 1;
 	 item->IsInBox = atoi(szNum);

 	 /* keep reading blocks till '\n' is found and
 	    count the number of bytes read */

 	 while(1) {
 	  lpT = str;
 	  while(((lpT - str) < (LONG) cbread)&&((*lpT) != '\n')&&((*lpT)!='\r')) {
 	   ++lpT ;
 	   ++count;
 	  }
 	  if((cbread < 512)||((*lpT) == '\r')||((*lpT) == '\n')) break;
 	  cbread      = _lread(hfile,str,512);
 	 }

 	 /* allocate the string sz */

 	 item->sz = (LPSTR) GlobalAllocPtr(GHND,count-chead);
 	 pos      = _llseek(hfile,pos+chead+icount+1,0);

 	 /* read item->sz */

 	 cbread   = _lread(hfile,item->sz,(UINT) (count-chead-1));

 	 /* to skip the '\r\n' character - reset the file pointer
 	    so that it points to the next item in the file and if
 	    the exepected '\r' or '\n' is not found return NULL */

 	 cbread     = _lread(hfile,szNum,1);
 	 if ((cbread) &&  (*(szNum) != '\r') && (*(szNum) != '\n')) {
 	  GlobalFreePtr(str);
 	  return NULL;
 	 }
 	 cbread     = _lread(hfile,szNum,1);
 	 if ((cbread) && (*(szNum) != '\r') && (*(szNum) != '\n')) {
 	  GlobalFreePtr(str);
 	  return NULL;
 	 }
 	 return item;

 	}
 	else {
 	 GlobalFreePtr(str);
 	 return NULL;
 	}
}


/*
 - LoadProfileSettings
 -
 * 	Purpose
 *		   Load settings from ini file 
 */


void LoadProfileSettings(LPVP lpvp)
{
    char     szNum[25];
	
    //Read Viewport Options from ini file
    lpvp->nLogLevel    =(int) GetPrivateProfileInt( lpvp->szTitle, 
    												"Global Nesting Level",1, 
    												"xvport.ini");
    												 
    lpvp->bAppendMode  =(int) GetPrivateProfileInt( lpvp->szTitle,
    											    "Append Loading Mode" ,0, 
    											    "xvport.ini");
                                                    
    lpvp->fInversion   =(int) GetPrivateProfileInt( lpvp->szTitle, 
    												"Inverted Mode" ,0, 
    												"xvport.ini");

    lpvp->nSaveLevel   =(int) GetPrivateProfileInt( lpvp->szTitle,
    											    "Saving Level",-1, 
    											    "xvport.ini");
                                                  
    lpvp->nTabCnt      =(int) GetPrivateProfileInt( lpvp->szTitle,  
    												"Tab Count", 4, 
    												"xvport.ini");
                                                    
    lpvp->bToolbar     =(int) GetPrivateProfileInt( lpvp->szTitle, 
    											    "Toolbar", 1, 
    											    "xvport.ini");
                                                    
    lpvp->rcPos.left   =(int) GetPrivateProfileInt(	lpvp->szTitle,  
    												"Left", 0, 
    												"xvport.ini");
                                                    
    lpvp->rcPos.top    =(int) GetPrivateProfileInt( lpvp->szTitle,  
    											    "Top", 0, 
    											    "xvport.ini");
                                                   
    lpvp->rcPos.right  =(int) GetPrivateProfileInt( lpvp->szTitle,  
    											    "Right", 0, 
    											    "xvport.ini");
                                                    
    lpvp->rcPos.bottom =(int) GetPrivateProfileInt( lpvp->szTitle,  
    											    "Bottom", 0, 
    											    "xvport.ini");

    lpvp->rcPos.top    =(int) GetPrivateProfileInt(	lpvp->szTitle,  
    											  	"Top", 0, 
    											  	"xvport.ini");
                                                    
    lpvp->rcPos.right  =(int) GetPrivateProfileInt(	lpvp->szTitle,  
    												"Right", 0, 
    												"xvport.ini");
                                                    
    lpvp->rcPos.bottom =(int) GetPrivateProfileInt(	lpvp->szTitle,  
    												"Bottom", 0, 
    												"xvport.ini");
                                                    
    lpvp->rcPos.top    =(int) GetPrivateProfileInt(	lpvp->szTitle,  
    												"Top", 0, 
    												"xvport.ini");
                                                    
    lpvp->rcPos.right  =(int) GetPrivateProfileInt(	lpvp->szTitle,  
    												"Right", 0, 
    												"xvport.ini");

    lpvp->lf.lfHeight = 0;
    lpvp->lf.lfWidth  = 0;
    lpvp->lf.lfWeight = 0;

#ifdef _WIN32
    if(GetPrivateProfileString( lpvp->szTitle, "Font Height",
    							"\0", szNum, 24, "xvport.ini"))
        lpvp->lf.lfHeight = atol(szNum);

    if(GetPrivateProfileString( lpvp->szTitle, "Font Width",
    							"\0", szNum, 24, "xvport.ini"))
        lpvp->lf.lfWidth  = atol(szNum);

    if(GetPrivateProfileString(lpvp->szTitle, "Font Weight",
    						   "\0", szNum, 24, "xvport.ini"))
        lpvp->lf.lfWeight  = atol(szNum);
#else

	if(GetPrivateProfileString(lpvp->szTitle, "Font Height",
							   "\0", szNum, 24, "xvport.ini"))
        lpvp->lf.lfHeight = atoi(szNum);

    if(GetPrivateProfileString(lpvp->szTitle, "Font Width",
    						   "\0", szNum, 24, "xvport.ini"))
        lpvp->lf.lfWidth  = atoi(szNum);

    if(GetPrivateProfileString(lpvp->szTitle, "Font Weight",
    						   "\0", szNum, 24, "xvport.ini"))
        lpvp->lf.lfWeight  = atoi(szNum);

#endif

    lpvp->lf.lfPitchAndFamily   =
    (BYTE)GetPrivateProfileInt(lpvp->szTitle,  
    						   "Font Family", 0,
    						   "xvport.ini");
                               
    GetPrivateProfileString(lpvp->szTitle,  "Font Name","\0",
    					    (LPSTR) (lpvp->lf.lfFaceName), 
    					    LF_FACESIZE, "xvport.ini");

    if( GetPrivateProfileString(lpvp->szTitle,  "Text Color", 
    							"\0", szNum, 24, "xvport.ini"))
        lpvp->clTextColor =  atol(szNum);
    else
        lpvp->clTextColor = 0;

    if(GetPrivateProfileString(lpvp->szTitle,  "Bkg Color", 
    						   "\0", szNum, 24, "xvport.ini"))
        lpvp->clBkgColor =   atol(szNum);
    else
        lpvp->clBkgColor = RGB(255, 255, 255);

}

/*
 - SaveProfileSettings
 -
 * 	Purpose
 *		   Write settings to ini file 
 */

VOID   SaveProfileSettings(LPVP lpvp)
{

    char szNum[15];
    
    //Write options to ini file

    WritePrivateProfileString(lpvp->szTitle, "Global Nesting Level" ,
                              itoa(lpvp->nLogLevel, szNum, 10),
                              "xvport.ini");

    WritePrivateProfileString(lpvp->szTitle, "Append Loading Mode" ,
                              itoa(lpvp->bAppendMode, szNum, 10),
                              "xvport.ini");

    WritePrivateProfileString(lpvp->szTitle, "Inversion Mode" ,
                              itoa(lpvp->fInversion, szNum, 10),
                              "xvport.ini");

    WritePrivateProfileString(lpvp->szTitle, "Saving Level",
                              itoa(lpvp->nSaveLevel, szNum, 10),
                              "xvport.ini");

    WritePrivateProfileString(lpvp->szTitle,  "Tab Count",
                              itoa(lpvp->nTabCnt, szNum, 10),
                              "xvport.ini");

    WritePrivateProfileString(lpvp->szTitle, "Toolbar",
                              itoa(lpvp->bToolbar, szNum, 10),
                              "xvport.ini");

    WritePrivateProfileString(lpvp->szTitle, "Left",
                              itoa(lpvp->rcPos.left, szNum, 10),
                              "xvport.ini");

    WritePrivateProfileString(lpvp->szTitle, "Top",
                              itoa(lpvp->rcPos.top, szNum, 10),
                              "xvport.ini");

    WritePrivateProfileString(lpvp->szTitle, "Right",
                              itoa(lpvp->rcPos.right, szNum, 10),
                              "xvport.ini");

    WritePrivateProfileString(lpvp->szTitle, "Bottom",
                              itoa(lpvp->rcPos.bottom, szNum, 10),
                              "xvport.ini");

    WritePrivateProfileString(lpvp->szTitle,  "Text Color",
                              ultoa(lpvp->clTextColor, szNum, 10),
                              "xvport.ini");

    WritePrivateProfileString(lpvp->szTitle,  "Bkg Color",
                              ultoa(lpvp->clBkgColor, szNum, 10),
                              "xvport.ini");

    WritePrivateProfileString(lpvp->szTitle, "Font Height",
                              ltoa(lpvp->lf.lfHeight, szNum, 10),
                              "xvport.ini");

    WritePrivateProfileString(lpvp->szTitle, "Font Width",
                              ltoa(lpvp->lf.lfWidth, szNum, 10),
                              "xvport.ini");

    WritePrivateProfileString(lpvp->szTitle, "Font Weight",
                              ltoa(lpvp->lf.lfWeight, szNum, 10),
                              "xvport.ini");

    WritePrivateProfileString(lpvp->szTitle, "Font Family",
                              itoa(lpvp->lf.lfPitchAndFamily, szNum, 10),
                              "xvport.ini");

    WritePrivateProfileString(lpvp->szTitle, "Font Name",
                              lpvp->lf.lfFaceName,
                              "xvport.ini");
}
