/*
 -  X V P O R T. C
 -
 *  Purpose:
 *      This file is the header for
 *      the eXpandable Viewport tool
 *
 *
 *  Author: Ben Kaizi-Lutu
 *
 *  Comments:
 *
 *      01/24/94    benkai    Original Version
 *		02/15/94    raos      File i/o tools
 *      mm/dd/yy    e-mail    Description.
 *
 */

 
//Define some things not definede for WIN16
#ifdef  WIN16
#define MAKEWPARAM(l, h)  ((WPARAM)MAKELONG(l, h))
#define MAX_PATH  256
#endif

//Define a structure for thread data
typedef struct _td
{
    DWORD       dwTId;	    	//Thread ID
    HANDLE      hT;				//Thread Handle
    HWND        hParent;		//Parent Window Handle
    HWND        hDlg;			//Dialog Handle
	HANDLE      hEvent;			//Manual reset event handle
    
} TD, FAR *LPTD; 



//Define a structure for a Viewport display item
typedef struct _vpitem
{
	LPSTR  		sz;				//Display String
	int    		nType;			//String type
	int    		nLevel;			//Indent level
	BOOL   		IsOpen;			//Is Item open
    BOOL   		IsInBox;		//Is Item in List box
	struct 		_vpitem * next;	//Next item ptr

} VPITEM, FAR * LPVPITEM;


//Define a structure for a Viewport instance
typedef struct _vp
{
	UINT 	  uID;				//Viewport ID
    LPTD 	  lptd;				//Threaddata ptr
    LPVPITEM  lpItemHead;		//Item List Head
    LPVPITEM  lpItemTail;		//Item List Tail
    int       nLogLevel ;	    //Logging level
	int       fInversion ;		//Inverted Display
	int       fToolbar   ;		//Toolbar  in view
	int       nSaveLevel;		//Saving Level
	int       nTabCnt;			//Tab-space expansion
	int       bToolbar;		    //Toolbar  in view
	int       bAppendMode;		//File Save mode
	RECT      rcPos;			//Viewport position
    COLORREF  clTextColor;		//Text color
    COLORREF  clBkgColor;		//Background color
    LOGFONT   lf;				//Logfont struct
    HFONT     hCurFont;         //Current Font handle
    HBRUSH    hLBBrush;         //List box bkg brush
	char      szFile[MAX_PATH] ;//File *.xvp
	char      szTitle[MAX_PATH];//Viewport Title
    struct   _vp * next;		//Next Viewport ptr
    
} VP, FAR * LPVP;    


#ifdef __cplusplus
extern "C" {
#endif

//Exported Entry points

BOOL    XVPInit( HWND);

UINT    XVPCreateViewport(LPSTR);

VOID    XVPDestroyViewport(UINT);   

VOID    XVPDeInit( );
				
VOID    XVPLog( UINT uID, int nLevel, LPSTR lpDesc );                     

VOID    XVPLogStatus(UINT uID,int nLevel, LPSTR lpDesc, BOOL IsFail);

VOID    XVPLogResult(UINT uID,int nLevel, LPSTR lpDesc, HRESULT hr );

VOID    XVPLogScode( UINT uID,int nLevel, LPSTR lpDesc, SCODE sc );

VOID    XVPSetLogLevel(UINT uID,int nLevel);

VOID    XVPPosition(UINT uID, int );

VOID    XVPReset(UINT uID);

VOID    XVPSetTitle(UINT uID, LPSTR szTitle);

VOID    XVPInvert(UINT uID);

VOID    XVPShow(UINT uID,int nShow);


#ifdef __cplusplus
}
#endif

//Internal utility functions' prototypes

VOID 		FreeVPIList(UINT );
BOOL    	FItemToFile(HFILE hf, LPVPITEM lptm);
LPVPITEM 	ItemFromFile(HFILE hfi); 
long 		ViewPortThread(LPVP  lpvp );
LPVP 		VPFromID( UINT uID);
LPVP 		VPFromHwnd( HWND hDlg);
BOOL 		CALLBACK ViewPortDlgProc(HWND , UINT , WPARAM , LPARAM );
BOOL 		CALLBACK OptionsDlgProc(HWND , UINT , WPARAM , LPARAM );
BOOL 		CALLBACK AboutDlgProc(HWND , UINT , WPARAM , LPARAM );
VOID   		SaveProfileSettings(LPVP);
VOID   		LoadProfileSettings(LPVP);


//Define Logging port levels
#define CASE_LEVEL       0
#define FUNCTION_LEVEL	 1
#define DETAIL_LEVEL     2

// define Dialog Positions 
#define	TOP_LEFT		1
#define	TOP_RIGHT		2
#define	CENTER			3
#define	BOTTOM_LEFT		4
#define	BOTTOM_RIGHT	5

// define log string types
#define FAIL 1
#define PASS 2
#define HEAD 4
#define TAIL 8

//define show flaga
#define XVP_HIDE 0
#define XVP_SHOW 1
#define XVP_MIN  2
#define XVP_MAX  3

//Define utility macros

#define NYI MessageBox( (HWND)NULL, \
                        "NYI!",\
                        "XVPORT",\
                        MB_ICONINFORMATION | MB_APPLMODAL | MB_OK )



#define MSGINF(szMsg) MessageBox( (HWND)NULL, \
                        szMsg,\
                        "Xvport - informer",\
                        MB_ICONINFORMATION | MB_APPLMODAL | MB_OK )


#define MSGSTOP(szMsg) MessageBox( (HWND)NULL, \
                        szMsg,\
                        "Xvport - error",\
                        MB_OK | MB_ICONSTOP)


